*********************************************************************
*                                                                   *
*    polarized radiatively generated lo and nlo parton densities    *
*                         and dis asymmetries                       *
*                                                                   *
*         m. gluck, e. reya, m. stratmann and w. vogelsang,         *
*                  (phys. rev. d53 (1996) 4775)                     *
*          problems/questions to vogelsang@v2.rl.ac.uk              *
*            or to strat@hal1.physik.uni-dortmund.de                *
*                                                                   *
*   input:   iset = number of the parton set :                      *
*              iset = 1  'standard' scenario, next-to-leading order *
*                        (ms-bar)                                   *
*                        (data file 'stdnloa1.grid' unit=11, to be  *
*                         defined by the user )                     *
*              iset = 2  'valence' scenario,  next-to-leading order *
*                        (ms-bar)                                   *
*                        (data file 'valnloa1.grid' unit=22, to be  *
*                         defined by the user )                     *
*              iset = 3  'standard' scenario, leading order         *
*                        (data file 'stdloa1.grid' unit=33, to be   *
*                         defined by the user )                     *
*              iset = 4  'valence' scenario,  leading order         *
*                        (data file 'valloa1.grid' unit=44, to be   *
*                         defined by the user )                     *
*                                                                   *
*            x  = bjorken-x       (between  1.e-4  and  1)          *
*            q2 = scale in gev**2 (between  0.4  and   1.e4)        *
*             (for values outside the allowed range the program     *
*              writes a warning and extrapolates to the x and       *
*              q2 values requested)                                 *
*                                                                   *
*   output:  uv = x * ( delta u - delta u(bar) ),                   *
*            dv = x * ( delta d - delta d(bar) ),                   *
*            qb = x * polarized light sea,                          *
*                 qb= x * ( (delta u(bar) + delta d(bar))/2 )       *
*            st = x * delta strange = x * delta strange(bar)        *
*            gl = x * delta gluon                                   *
*                                                                   *
*          (  for the parton distributions always x times           *
*                   the distribution is returned   )                *
*                                                                   *
*            a1p = a1-proton                                        *
*            a1n = a1-neutron                                       *
*            a1d = a1-deuteron                                      *
*                                                                   *
*            the sets are the result of a combined fit to           *
*            data for the spin asymmetries a_1 (p,n,d)              *
*                                                                   *
*            note: no charm is included                             *
*                                                                   *
*   common:  the main program or the calling routine has to have    *
*            a common block  common / intini / iini , and  iini     *
*            has always to be zero when parpol is called for the    *
*            first time or when 'iset' has been changed.            *
*                                                                   *
*********************************************************************
*
      subroutine parpol (x, q2, uv, dv, qb, st, gl, a1p, a1n,
     1                                                    a1d)
      implicit double precision (a-h,o-z)
      character path*80
      parameter (npart=8, nx=42, nq=25, narg=2)
      dimension xuvf(nx,nq), xdvf(nx,nq), xqbf(nx,nq),
     1          xsf(nx,nq), xgf(nx,nq), xapf(nx,nq), xanf(nx,nq),
     2          xadf(nx,nq), parton (npart,nq,nx-1),
     3          qs(nq), xb(nx), xt(narg), na(narg), arrf(nx+nq)
      common / intini / iini
      save xuvf, xdvf, xqbf, xsf, xgf, xapf, xanf, xadf, na, arrf
*...bjorken-x and q**2 values of the grid :
       data qs / 0.4d0, 0.5d0, 0.6d0, 0.75d0, 1.0d0,
     1           1.25d0, 1.5d0, 2.d0, 2.5d0,
     1           4.0d0, 6.4d0, 1.0d1, 1.5d1, 2.5d1, 4.0d1, 6.4d1,
     2           1.0d2, 1.8d2, 3.2d2, 5.8d2, 1.0d3, 1.8d3,
     3           3.2d3, 5.8d3, 1.0d4 /
       data xb / 1.d-4, 1.5d-4, 2.2d-4, 3.2d-4, 4.8d-4, 7.d-4,
     2           1.d-3, 1.5d-3, 2.2d-3, 3.2d-3, 4.8d-3, 7.d-3,
     3           1.d-2, 1.5d-2, 2.2d-2, 3.2d-2, 5.0d-2, 7.5d-2,
     4           0.1, 0.125, 0.15, 0.175, 0.2, 0.225, 0.25, 0.275,
     5           0.3, 0.325, 0.35, 0.375, 0.4, 0.45,  0.5, 0.55,
     6           0.6, 0.65,  0.7,  0.75,  0.8, 0.85,  0.9, 1.0 /
*...check of x and q2 values :
       if ( (x.lt.1.0d-4) .or. (x.gt.1.0d0) ) then
           write(6,91)
  91       format (2x,'parton interpolation: x out of range')
c          goto 60
       endif
       if ( (q2.lt.0.4d0) .or. (q2.gt.1.d4) ) then
           write(6,92)
  92       format (2x,'parton interpolation: q2 out of range')
c          goto 60
       endif
*...initialization :
*    selection and reading of the grid :
*    file - no. = 11 for nlo 'standard' scenario ( first number in the
*                                                  grid: 1.040e-03 )
*    file - no. = 22 for nlo 'valence'  scenario ( first number in the
*                                                  grid: 9.740e-04 )
*    file - no. = 33 for  lo 'standard' scenario ( first number in the
*                                                  grid: 1.731e-03 )
*    file - no. = 44 for  lo 'valence'  scenario ( first number in the
*                                                  grid: 1.846e-03 )
      if (iini.ne.0) goto 16

      iiread=33
      call getenv('RENESANCE_ROOT',path)
      open(unit=33,file=TRIM(ADJUSTL(path))//'/share/renesance/pdf/stdloa1.gri',status='old')

c
       do 15 m = 1, nx-1
       do 15 n = 1, nq
       read(iiread,90) parton(1,n,m), parton(2,n,m), parton(3,n,m),
     1                 parton(4,n,m), parton(5,n,m), parton(6,n,m),
     2                 parton(7,n,m), parton(8,n,m)
  90   format (8(1pe10.3))
  15   continue
c
      iini = 1
*....arrays for the interpolation subroutine :
      do 10 iq = 1, nq
      do 20 ix = 1, nx-1
        xb0 = xb(ix)
        xb1 = 1.d0-xb(ix)
        xuvf(ix,iq) = parton(1,iq,ix) / (xb1**3 * xb0**0.7)
        xdvf(ix,iq) = parton(2,iq,ix) / (xb1**4 * xb0**0.6)
        xqbf(ix,iq) = parton(3,iq,ix) / (xb1**7 * xb0**0.3)
        xsf(ix,iq)  = parton(4,iq,ix) / (xb1**7 * xb0**0.3)
        xgf(ix,iq)  = parton(5,iq,ix) / (xb1**10 * xb0**0.3)
        xapf(ix,iq)  = parton(6,iq,ix) / xb0
        xanf(ix,iq)  = parton(7,iq,ix) / xb0
        xadf(ix,iq)  = parton(8,iq,ix) / xb0
  20  continue
        xuvf(nx,iq) = 0.d0
        xdvf(nx,iq) = 0.d0
        xqbf(nx,iq) = 0.d0
        xsf(nx,iq)  = 0.d0
        xgf(nx,iq)  = 0.d0
        xapf(nx,iq)  = 0.d0
        xanf(nx,iq)  = 0.d0
        xadf(nx,iq)  = 0.d0
  10  continue
      na(1) = nx
      na(2) = nq
      do 30 ix = 1, nx
        arrf(ix) = dlog(xb(ix))
  30  continue
      do 40 iq = 1, nq
        arrf(nx+iq) = dlog(qs(iq))
  40  continue
  16  continue
*...interpolation :
      xt(1) = dlog(x)
      xt(2) = dlog(q2)
      uv = dfint(narg,xt,na,arrf,xuvf) * (1.d0-x)**3 * x**0.7
      dv = dfint(narg,xt,na,arrf,xdvf) * (1.d0-x)**4 * x**0.6
      qb = dfint(narg,xt,na,arrf,xqbf) * (1.d0-x)**7 * x**0.3
      st = dfint(narg,xt,na,arrf,xsf)  * (1.d0-x)**7 * x**0.3
      gl = dfint(narg,xt,na,arrf,xgf)  * (1.d0-x)**10 * x**0.3
      a1p = dfint(narg,xt,na,arrf,xapf)  * x
      a1n = dfint(narg,xt,na,arrf,xanf)  * x
      a1d = dfint(narg,xt,na,arrf,xadf)  * x
 60   return
      end
*
*...cern library routine e104 (interpolation) :
*

      function dfint(narg,arg,nent,ent,table)
      implicit double precision (a-h,o-z)
      dimension arg(2),nent(2),ent(67),table(1050)
      dimension d(2),ncomb(2),ient(2)
      kd=1
      m=1
      ja=1
         do 5 i=1,narg
      ncomb(i)=1
      jb=ja-1+nent(i)
         do 2 j=ja,jb
      if (arg(i).le.ent(j)) go to 3
    2 continue
      j=jb
    3 if (j.ne.ja) go to 4
      j=j+1
    4 jr=j-1
      d(i)=(ent(j)-arg(i))/(ent(j)-ent(jr))
      ient(i)=j-ja
      kd=kd+ient(i)*m
      m=m*nent(i)
    5 ja=jb+1
      dfint=0.d0
   10 fac=1.d0
      iadr=kd
      ifadr=1
         do 15 i=1,narg
      if (ncomb(i).eq.0) go to 12
      fac=fac*(1.d0-d(i))
      go to 15
   12 fac=fac*d(i)
      iadr=iadr-ifadr
   15 ifadr=ifadr*nent(i)
      dfint=dfint+fac*table(iadr)
      il=narg
   40 if (ncomb(il).eq.0) go to 80
      ncomb(il)=0
      if (il.eq.narg) go to 10
      il=il+1
         do 50  k=il,narg
   50 ncomb(k)=1
      go to 10
   80 il=il-1
      if(il.ne.0) go to 40
      return
      end

      subroutine parpolinit()
         implicit double precision (a-h,o-z)
         common / intini / iini

         iini = 0
         x=0.5
         q2=4.
         call parpol (x, q2, uv, dv, qb, st, gl, a1p, a1n, a1d)
      end