#ifndef PARTONDIDTRIBUTIONSGRSV96_H
#define PARTONDIDTRIBUTIONSGRSV96_H

#include "General.h"
#include "PartonDistributions.h"

namespace SANC {

class PartonDistributionsGRSV96 : public PDF {
public:
  int pdfID() { return 20002; };

private:
  double XF;
  void _init() {}

  double _xfA(int i, double x, double Q2) {
    switch (i) {
    case -3:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case -2:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case -1:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 1:
      XF = -0.3392 * pow(x, 0.) * dmdb(x, Q2) -
           1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 2:
      XF = 0.6392 * pow(x, 0.16) * umub(x, Q2) -
           1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 3:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    default:
      XF = 0.;
      break;
    }
    return XF;
  }

  double _xfB(int i, double x, double Q2) {
    switch (i) {
    case -3:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case -2:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case -1:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 1:
      XF = -0.3392 * pow(x, 0.) * dmdb(x, Q2) -
           1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 2:
      XF = 0.6392 * pow(x, 0.16) * umub(x, Q2) -
           1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    case 3:
      XF = -1.348 * pow(x, 0.93) * pow(1. - x, 0.09) * dbpub(x, Q2) / 2.;
      break;
    default:
      XF = 0.;
      break;
    }
    return XF;
  }
  double _AlfasQ(double Q) { return 0.; }
  double _Q2min() { return 0.4; }
  double _xmin() { return 1.e-5; }
  double _s(double Q2) { return log(log(Q2 / pow2(0.232)) / log(0.23 / pow2(0.232))); }
  double umub(double x, double Q2) {
    double s = _s(Q2);
    double a = 0.590 - 0.024 * s;
    double b = 0.131 + 0.063 * s;
    double N = 2.284 + 0.802 * s + 0.055 * pow2(s);
    double A = -0.449 - 0.138 * s - 0.076 * pow2(s);
    double B = 0.213 + 2.669 * s - 0.728 * pow2(s);
    double C = 8.854 - 9.135 * s + 1.979 * pow2(s);
    double D = 2.997 + 0.753 * s - 0.076 * pow2(s);
    return N * pow(x, a) * (1. + A * pow(x, b) + B * x + C * pow(x, 1.5)) * pow(1. - x, D);
  }
  double dmdb(double x, double Q2) {
    double s = _s(Q2);
    double a = 0.376;
    double b = 0.486 + 0.062 * s;
    double N = 0.371 + 0.083 * s + 0.039 * pow2(s);
    double A = -0.509 + 3.310 * s - 1.248 * pow2(s);
    double B = 12.41 - 10.52 * s + 2.267 * pow2(s);
    double C = 6.373 - 6.208 * s + 1.418 * pow2(s);
    double D = 3.691 + 0.799 * s - 0.071 * pow2(s);
    return N * pow(x, a) * (1. + A * pow(x, b) + B * x + C * pow(x, 1.5)) * pow(1. - x, D);
  }
  double dbmub(double x, double Q2) {
    double s = _s(Q2);
    double a = 0.409 - 0.005 * s;
    double b = 0.799 + 0.071 * s;
    double N = 0.082 + 0.014 * s + 0.008 * pow2(s);
    double A = -38.07 + 36.13 * s - 0.656 * pow2(s);
    double B = 90.31 - 74.15 * s + 7.645 * pow2(s);
    double C = 0.;
    double D = 7.486 + 1.217 * s - 0.159 * pow2(s);
    return N * pow(x, a) * (1. + A * pow(x, b) + B * x + C * pow(x, 1.5)) * pow(1. - x, D);
  }
  double dbpub(double x, double Q2) {
    double s = _s(Q2);
    double alpha = 1.451;
    double beta = 0.271;
    double a = 0.410 - 0.232 * s;
    double b = 0.534 - 0.457 * s;
    double A = 0.890 - 0.140 * s;
    double B = -0.981;
    double C = 0.320 + 0.683 * s;
    double D = 4.752 + 1.164 * s + 0.286 * pow2(s);
    double E = 4.119 + 1.713 * s;
    double Ep = 0.682 + 2.978 * s;
    return (pow(x, a) * (A + B * x + C * pow2(x)) * pow(log(1. / x), b) +
            pow(s, alpha) * exp(-E + sqrt(Ep * pow(s, beta) * log(1. / x)))) *
           pow(1. - x, D);
  }
  double spdf(double x, double Q2) {
    double s = _s(Q2);
    double alpha = 0.914;
    double beta = 0.577;
    double a = 1.798 - 0.596 * s;
    double A = -5.548 + 3.669 * sqrt(s) - 0.616 * s;
    double B = 18.92 - 16.73 * sqrt(s) + 5.168 * s;
    double D = 6.379 - 0.350 * s + 0.142 * pow2(s);
    double E = 3.981 + 1.638 * s;
    double Ep = 6.402;
    return pow(s, alpha) / pow(log(1. / x), a) * (1. + A * sqrt(x) + B * x) * pow(1. - x, D) *
           exp(-E + sqrt(Ep * pow(s, beta) * log(1. / x)));
  }
};
} // namespace SANC

#endif // PARTONDIDTRIBUTIONSGRSV96_H
