************************************************************************
* This is the FORTRAN module (cc_br_1413_1615) to calculate EW Born
* cross section and soft photon Bremsstrahlung
* for the anti-dn + up -> mo^+ + mn process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2011.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine cc_br_1413_1615_pol (qs,ts,us,bornpp,bornpm,bornmp,bornmm,softpp,softpm,softmp,softmm,hard)
      implicit none!
      include 's2n_declare.h'

      integer iborn_
      real*8 bornpp,bornpm,bornmp,bornmm
      real*8 softpp,softpm,softmp,softmm

      real*8 softvirtisr,softvirtfsr,softvirtifi
      real*8 ln1sdn,ln1sup,ln1smo

      real*8 betasmoc,ibetasmoc,sqrtlsmoc
      real*8 cpl,cmi
      real*8 kappaw,qw
      complex*16 chiws,chiwsc

      cmw2 = mw2
      qw   = 1d0

      s = -qs
      t = -ts
      u = -us

      cosf = (u-t)/(s-mmo**2)
      sinf = dsqrt(1d0-cosf**2)

      cpl = 1d0+cosf
      cmi = 1d0-cosf

      sqrtlsmoc = dabs(s-mmo**2)
      betasmoc = (s-mmo**2)/(s+mmo**2)
      ibetasmoc = 1d0/betasmoc

      kappaw = gf*rwm2/(sqrt(2d0)*4d0*pi)/alpha
      if (irun.eq.0) then
         chiws = 4d0*kappaw*s/(s-cmw2)
      else
         chiws = 4d0*kappaw*s/(s-dcmplx(mw**2,-s*ww/mw))
      endif
      chiwsc = dconjg(chiws)

      iborn_ = iborn
      iborn  = 1
      call cc_si_1413_1615_pol (s,t,u,bornpp,bornpm,bornmp,bornmm)
      iborn  = iborn_
      if (bornpp.ne.bornpp) print *,'Born pp NAN!!!',s,t,u
      if (bornpm.ne.bornpm) print *,'Born pm NAN!!!',s,t,u
      if (bornmp.ne.bornmp) print *,'Born mp NAN!!!',s,t,u
      if (bornmm.ne.bornmm) print *,'Born mm NAN!!!',s,t,u
      if (iborn.eq.1) return

      ln1sdn = log(s/mdn**2)-1d0
      ln1sup = log(s/mup**2)-1d0
      ln1smo = log(s/mmo**2)-1d0

      softvirtisr=
     & +1d0/pi*alpha*qdn**2*(3d0/2+3d0/4*ln1sdn-3d0/4*log(s/thmu2))
     & +1d0/pi*alpha*qup**2*(3d0/2+3d0/4*ln1sup-3d0/4*log(s/thmu2))
     & +log(4*omega**2/s)*1d0/pi*alpha*qdn**2*(1d0/2*ln1sdn)
     & +log(4*omega**2/s)*1d0/pi*alpha*qup**2*(1d0/2*ln1sup)
     & +log(4*omega**2/s)*1d0/pi*alpha*qw**2*(-1d0/2)
     & +ddilog(1d0)*1d0/pi*alpha*qup*qdn*(2)

      softvirtfsr=
     & +1d0/pi*alpha*qmo**2*(1+1d0/4*ln1smo+1d0/2*ln1smo*ibetasmoc+1d0/
     & 2*ibetasmoc-3d0/4*log(s/thmu2))
     & +log(4*omega**2/s)*1d0/pi*alpha*qmo**2*(1d0/2*ln1smo)
     & +log(4*omega**2/s)*1d0/pi*alpha*qw**2*(-1d0/2)

      softvirtifi=
     & +log(mmo**2/s+us/s)**2*1d0/pi*alpha*qup*qmo*(2)
     & +log(mmo**2/s+ts/s)**2*1d0/pi*alpha*qdn*qmo*(-2)
     & +log(us/s)*log(mmo**2/s+us/s)*1d0/pi*alpha*qup*qmo*(-2)
     & +log(us/s)**2*1d0/pi*alpha*qup*qmo*(1d0/2)
     & +log(ts/s)*log(mmo**2/s+ts/s)*1d0/pi*alpha*qdn*qmo*(2)
     & +log(ts/s)**2*1d0/pi*alpha*qdn*qmo*(-1d0/2)
     & +log(4*omega**2/s)*1d0/pi*alpha*qw**2*(1)
     & +log(4*omega**2/s)*log(mmo**2/s+us/s)*1d0/pi*alpha*qup*qmo*(-1)
     & +log(4*omega**2/s)*log(mmo**2/s+ts/s)*1d0/pi*alpha*qdn*qmo*(1)
     & +ddilog(-1d0/ts*mmo**4/s-mmo**2/s)*1d0/pi*alpha*qdn*qmo*(-1)
     & +ddilog(-1d0/us*mmo**4/s-mmo**2/s)*1d0/pi*alpha*qup*qmo*(1)
     & +ddilog(-mmo**2/s+1/(mmo**2+us)*mmo**2)*1d0/pi*alpha*qup*qmo*(-1
     & )
     & +ddilog(-mmo**2/s+1/(mmo**2+ts)*mmo**2)*1d0/pi*alpha*qdn*qmo*(1)
     & +ddilog(1/(mmo**2+us)*mmo**2)*1d0/pi*alpha*qup*qmo*(1)
     & +ddilog(1/(mmo**2+ts)*mmo**2)*1d0/pi*alpha*qdn*qmo*(-1)
     & +ddilog(-1/(mmo**4+2*us*mmo**2+us**2)*ts*us)*1d0/pi*alpha*qup*qm
     & o*(1)
     & +ddilog(-1/(mmo**4+2*ts*mmo**2+ts**2)*ts*us)*1d0/pi*alpha*qdn*qm
     & o*(-1)
     & +ddilog(1d0)*1d0/pi*alpha*qdn*qmo*(-1)
     & +ddilog(1d0)*1d0/pi*alpha*qup*qmo*(1)

      if (iqed.eq.1) then
         soft = softvirtisr+softvirtifi+softvirtfsr
      elseif(iqed.eq.2) then
         soft = softvirtisr
      elseif(iqed.eq.3) then
         soft = softvirtifi
      elseif(iqed.eq.4) then
         soft = softvirtfsr
      elseif(iqed.eq.5) then
         soft = softvirtifi+softvirtfsr
      elseif(iqed.eq.6) then
         soft = softvirtisr+softvirtifi
      endif
      soft = soft*cfprime

      softpp = soft*bornpp
      softpm = soft*bornpm
      softmp = soft*bornmp
      softmm = soft*bornmm

      return
      end
