************************************************************************
* This is the FORTRAN module (nc_br_1414_1616) to calculate EW Born
* cross section and soft photon Bremsstrahlung
* for the anti-dn + dn -> mo^+ + mo^- process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2012.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine nc_br_1414_1616_pol (qs,ts,us,bornpp,bornpm,bornmp,bornmm,softpp,softpm,softmp,softmm,hard)
      implicit none!
      include 's2n_declare.h'

      integer iborn_
      real*8 bornpp,bornpm,bornmp,bornmm
      real*8 softpp,softpm,softmp,softmm
      real*8 lnlaom,cpl,cmi
      real*8 a0s,v0s,vaz
      real*8 a1z,a2z,v1z,v2z,vas
      real*8 softisr,softfsr,softifi
      real*8 kappaz
      complex*16 chizs,chizsc,chizspr,chizsprc
      real*8 betasmomo,ibetasmomo
      real*8 jintqsmomo

      cmz2 = mz2
      s = -qs
      t = -ts
      u = -us

      betasmomo = sqrt(1d0-4d0*mmo**2/s)
      ibetasmomo = 1d0/betasmomo

      iborn_ = iborn
      iborn  = 1
      call nc_si_1414_1616_pol (s,t,u,bornpp,bornpm,bornmp,bornmm)
      born = 0.25*(bornpp+bornpm+bornmp+bornmm)
      iborn  = iborn_
      if (bornpp.ne.bornpp) print *,'Born pp NAN!!!',s,t,u
      if (bornpm.ne.bornpm) print *,'Born pm NAN!!!',s,t,u
      if (bornmp.ne.bornmp) print *,'Born mp NAN!!!',s,t,u
      if (bornmm.ne.bornmm) print *,'Born mm NAN!!!',s,t,u
      if (iborn.eq.1) return

      lnlaom = log(4d0*omega**2/tlmu2)
      jintqsmomo = log((1d0-betasmomo)/(1d0+betasmomo))

      softisr=
     & +pi*alpha*qdn**2*(-1d0/3)
     & +lnlaom/pi*alpha*qdn**2*(-1)
     & +lnlaom*log(mdn**2/s)/pi*alpha*qdn**2*(-1)
     & +log(mdn**2/s)/pi*alpha*qdn**2*(-1)
     & +log(mdn**2/s)**2/pi*alpha*qdn**2*(-1d0/2)

      softfsr=
     & +ibetasmomo/pi*alpha*qmo**2*(-jintqsmomo-1d0/2*jintqsmomo**2
     & +jintqsmomo**2*mmo**2/s)
     & +ibetasmomo*pi*alpha*qmo**2*(-1d0/3+2d0/3*mmo**2/s)
     & +ibetasmomo*lnlaom/pi*alpha*qmo**2*(-jintqsmomo+2*jintqsmomo
     & *mmo**2/s)
     & +ibetasmomo*log(2-2/(1+betasmomo))/pi*alpha*qmo**2*(2*jintqs
     & momo-4*jintqsmomo*mmo**2/s)
     & +ibetasmomo*ddilog(-1+2/(1+betasmomo))/pi*alpha*qmo**2*(2-4*
     & mmo**2/s)
     & +lnlaom/pi*alpha*qmo**2*(-1)

      softifi=
     & +lnlaom*log(1/(mmo**2+us)*mmo**2+1/(mmo**2+us)*ts)/pi*alpha*
     & qdn*qmo*(2)
     & +log(1+1d0/qs/ts*mmo**4+2/qs*mmo**2+1d0/qs*ts)*log(-1/(mmo**4+2*t
     & s*mmo**2+ts**2)*qs*ts)/pi*alpha*qdn*qmo*(2)
     & +log(1+1d0/qs/us*mmo**4+2/qs*mmo**2+1d0/qs*us)*log(-1/(mmo**4+2*u
     & s*mmo**2+us**2)*qs*us)/pi*alpha*qdn*qmo*(-2)
     & +log(1-2/(1+betasmomo)/(mmo**2+us)*mmo**2)/pi*alpha*qdn*qmo*
     & (-2*jintqsmomo)
     & +log(1-2/(1+betasmomo)/(mmo**2+us)*mmo**2)**2/pi*alpha*qdn*q
     & mo*(1)
     & +log(1-2/(1+betasmomo)/(mmo**2+ts)*mmo**2)/pi*alpha*qdn*qmo*
     & (2*jintqsmomo)
     & +log(1-2/(1+betasmomo)/(mmo**2+ts)*mmo**2)**2/pi*alpha*qdn*q
     & mo*(-1)
     & +log(-1/(mmo**4+2*us*mmo**2+us**2)*qs*us)**2/pi*alpha*qdn*qm
     & o*(-1)
     & +log(-1/(mmo**4+2*ts*mmo**2+ts**2)*qs*ts)**2/pi*alpha*qdn*qm
     & o*(1)
     & +ddilog(1-2/(1+betasmomo)/(mmo**2+us)*us)/pi*alpha*qdn*qmo*(
     & -2)
     & +ddilog(1-2/(1+betasmomo)/(mmo**2+ts)*ts)/pi*alpha*qdn*qmo*(
     & 2)
     & +ddilog(-1d0/qs/ts*mmo**4-2/qs*mmo**2-1d0/qs*ts)/pi*alpha*qd
     & n*qmo*(-2)
     & +ddilog(-1d0/qs/us*mmo**4-2/qs*mmo**2-1d0/qs*us)/pi*alpha*qd
     & n*qmo*(2)
     & +ddilog(-1/(-mmo**2+us+betasmomo*mmo**2+betasmomo*us)*mmo**2-1/(-
     & mmo**2+us+betasmomo*mmo**2+betasmomo*us)*us+1/(-mmo**2+us+betasmo
     & mo*mmo**2+betasmomo*us)*betasmomo*mmo**2+1/(-mmo**2+us+betasmomo*
     & mmo**2+betasmomo*us)*betasmomo*us)/pi*alpha*qdn*qmo*(2)
     & +ddilog(-1/(-mmo**2+ts+betasmomo*mmo**2+betasmomo*ts)*mmo**2-1/(-
     & mmo**2+ts+betasmomo*mmo**2+betasmomo*ts)*ts+1/(-mmo**2+ts+betasmo
     & mo*mmo**2+betasmomo*ts)*betasmomo*mmo**2+1/(-mmo**2+ts+betasmomo*
     & mmo**2+betasmomo*ts)*betasmomo*ts)/pi*alpha*qdn*qmo*(-2)

      if (iqed.eq.1) then
         soft = softisr+softfsr+softifi
      elseif(iqed.eq.2) then
         soft = softisr
      elseif(iqed.eq.3) then
         soft = softifi
      elseif(iqed.eq.4) then
         soft = softfsr
      elseif(iqed.eq.5) then
         soft = softifi+softfsr
      elseif(iqed.eq.6) then
         soft = softisr+softifi
      endif
      soft = soft*cfprime

      softpp = soft*bornpp
      softpm = soft*bornpm
      softmp = soft*bornmp
      softmm = soft*bornmm

      return
      end
