************************************************************************
* This is the FORTRAN module (nc_ha_12_12__16_16) to calculate differential
* hard photon Bremsstrahlung for the el^+ + el^- -> mu^+ + mu^- process.
*
* Copyright (C), SANC Project Team, 2002-2022.
*
* http://sanc.jinr.ru/
* E-mail: <sanc@jinr.ru>; <sanc-team@cern.ch>
************************************************************************

      subroutine nc_ha_1313_1616_pol (s,spr,ph4,costh4,costh5,hardpp,hardpm,hardmp,hardmm)
      implicit none!
      include 's2n_declare.h'
      real*8 ph4
      integer*4 h1,h2,h3,h4,h5,j1,j2,i
      real*8 hardpp,hardpm,hardmp,hardmm
      real*8 sqs,sqspr
      real*8 costh4,costh5
      complex*16  ampISR(2,2,2,2,2), ampFSR(2,2,2,2,2)
      complex*16  hel_ampISR(2,2,2,2,2), hel_ampFSR(2,2,2,2,2)
      real*8 amp2(2,2,2,2,2),hard2(2,2)
      real*8 betai
      real*8 p3r1(5),p4r1(5),p3r2(5),p4r2(5)
      real*8 M_I,M_F
      real*8 i3f2,i3f3
      real*8 abs2,abs2p

      complex*16 DZZisr(2,2), DZZfsr(2,2)
      complex*16 DLL2_3_QQ, DRR2_3_QQ, DRL2_3_QQ, DRL3_2_QQ
      complex*16 cc1(2,2), cc2(2,2)
      real*8 p1(5), p2(5), p3(5), p4(5), p5(5);

#ifndef GENERATOR
      real*8 mp(5)

      mp = (/mel, mel, mmo, mmo, 0d0/)
      ! toMomenta for cos(theta) & toMomenta1 for rapidity
      print *,'before', pg1,pg3,pg4,pg5
      call toMomenta(s,spr,ph4,costh4,costh5,mp)
      print *,'after', pg1,pg3,pg4,pg5
      
#endif

!-----------------------------------------------------------------------
!--- p(1) = px, p(2) = py, p(3) = pz, p(4) = e, p(5) = m ---------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!--- 4-momenta of particles in the laboratory reference frame as input
      p1 = pg1
      p2 = pg2
      p3 = pg3
      p4 = pg4
      p5 = pg5
c      print *,'mod p1', p1
c      print *,'mod p2', p2
c      print *,'mod p3', p3
c      print *,'mod p4', p4
c      print *,'mod p5', p5
!--- But we work with all momenta to zero setup
!--- Do all momenta incoming
      do i = 1,4
        p3(i) = -p3(i)
        p4(i) = -p4(i)
        p5(i) = -p5(i)
      enddo

      M_I = p1(5)
      M_F = p3(5)
c      M_I = mup
c      M_F = mmo
      i3f2 = 1d0
      i3f3 = -1d0
      sqs = sqrt(s)
      sqspr = sqrt(spr)
      betai = sqrt((sqs-2d0*M_I)*(sqs+2d0*M_I))/sqs
      betaf = sqrt((sqspr-2d0*M_F)*(sqspr+2d0*M_F))/sqspr

      call S_CC(p1,p5,-1d0,cc1)
      call S_CC(p2,p5,+1d0,cc2)

      DZZisr(2,2)=DLL2_3_QQ(spr,qup,qmo,i3f2,i3f3);
      DZZisr(1,2)=DRL2_3_QQ(spr,qup,qmo,i3f2,i3f3);
      DZZisr(2,1)=DRL3_2_QQ(spr,qup,qmo,i3f2,i3f3);
      DZZisr(1,1)=DRR2_3_QQ(spr,qup,qmo,i3f2,i3f3);

      DZZfsr(2,2)=DLL2_3_QQ(s,qup,qmo,i3f2,i3f3);
      DZZfsr(1,2)=DRL3_2_QQ(s,qup,qmo,i3f2,i3f3); ! note the transposition compared to ISR
      DZZfsr(2,1)=DRL2_3_QQ(s,qup,qmo,i3f2,i3f3); ! note the transposition compared to ISR
      DZZfsr(1,1)=DRR2_3_QQ(s,qup,qmo,i3f2,i3f3);

      call nc_ha_f1_f1__f2_f2_isrSA(p1,p2,p3,p4,p5,qup,DZZisr,ampISR)
      call nc_ha_f1_f1__f2_f2_isrSA(p3,p4,p1,p2,p5,qmo,DZZfsr,ampFSR)

      do h1=1,2
        do h2=1,2
          do h3=1,2
            do h4=1,2
              do h5=1,2
                hel_ampISR(h1,h2,h3,h4,h5)=0d0
                hel_ampFSR(h1,h2,h3,h4,h5)=0d0
                do j1 = 1,2
                  do j2 = 1,2
                    hel_ampISR(h1,h2,h3,h4,h5)=hel_ampISR(h1,h2,h3,h4,h5)
     &              +cc1(j1,h1)*cc2(j2,h2)*(ampISR(j1,j2,h3,h4,h5))
                    hel_ampFSR(h1,h2,h3,h4,h5)=hel_ampFSR(h1,h2,h3,h4,h5)
     &              +cc1(j1,h1)*cc2(j2,h2)*(ampFSR(h3,h4,j1,j2,h5))
                  enddo
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo

! phase space volume
      volum =  1d0/2048d0/pi**4*(s-spr)/s**2*betaf/betai*conhc

! amplitudes squared
      do h1=1,2
        do h2=1,2
          hard2(h1,h2) = 0d0
          do h3=1,2
            do h4=1,2
              do h5=1,2
                amp2(h1,h2,h3,h4,h5) = abs2p(hel_ampISR(h1,h2,3-h3,3-h4,3-h5),hel_ampFSR(h1,h2,3-h3,3-h4,3-h5))
                hard2(h1,h2) = hard2(h1,h2) + amp2(h1,h2,h3,h4,h5)
              enddo
            enddo
          enddo
        enddo
      enddo

      hardpp = hard2(1,1)*volum*e**6*cfprime/3d0
      hardpm = hard2(1,2)*volum*e**6*cfprime/3d0
      hardmp = hard2(2,1)*volum*e**6*cfprime/3d0
      hardmm = hard2(2,2)*volum*e**6*cfprime/3d0

c      print *,'mod volum',volum*e**6*cfprime/3d0
c      print *,'mod cfprime',cfprime
c      print *,'mod e',e
c      print *,'mod betaf',betaf
c      print *,'mod betai',betai
c      print *,'mod M_I',M_I
c      print *,'mod M_F',M_F
c      print *,'mod hardpp', hardpp
c      print *,'mod hardpm', hardpm
c      print *,'mod hardmp', hardmp
c      print *,'mod hardmm', hardmm

c      if (hard.ne.hard) then
c        print *,"NAN hard:",s,spr
c        hard = 0d0
c      endif
      return
      end
