#include "Info.h"
#include "Integrand.h"
#include "LHEF.h"
#include "ModulesInterface.h"
#include "PartonDidtributionsLHAPDF.h"
#include "Process_pp.h"
#include "TMath.h"
#include "Vec4.h"
#include <cmath>
#include <fstream>
#include <iostream>
using namespace std;
using namespace SANC;
using namespace SANCmodules_pp;
using namespace SANCmodulesVar;

Integrand_pp::Integrand_pp(SANC::Info &_info, SANC::Process &_proc, PDF *_pdf, PDF * _pdfpol, int type)
    : info(_info), proc(_proc), pdf(_pdf), pdfpol(_pdfpol) {
  m_Type = type;
  Mf1 = proc.Mf1;
  Mf2 = proc.Mf2;
  Mf3 = proc.Mf3;
  Mf4 = proc.Mf4;
  Mf1_2 = proc.Mf1_2;
  Mf2_2 = proc.Mf2_2;
  Mf3_2 = proc.Mf3_2;
  Mf4_2 = proc.Mf4_2;
  sm = proc.sm;
  dm = proc.dm;
  P_com[0] = &proc.event.P_com(0);
  P_com[1] = &proc.event.P_com(1);
  P_com[2] = &proc.event.P_com(2);
  P_com[3] = &proc.event.P_com(3);
  P_com[4] = &proc.event.P_com(4);
  P_com[5] = &proc.event.P_com(5);
  P_lab[0] = &proc.event.P_lab(0);
  P_lab[1] = &proc.event.P_lab(1);
  P_lab[2] = &proc.event.P_lab(2);
  P_lab[3] = &proc.event.P_lab(3);
  P_lab[4] = &proc.event.P_lab(4);
  P_lab[5] = &proc.event.P_lab(5);
  *P_com[0] = Vec4();
  *P_lab[0] = Vec4();
}

double Integrand_pp::Density(int nDim, double *Xarg) {
  double rho;
  rho = 1e-100;
  if (m_Type == 10)
    rho = BornH(nDim, Xarg);
  else if (m_Type == 11)
    rho = VirtH(nDim, Xarg);
  else if (m_Type == 12)
    rho = HardH(nDim, Xarg);
  else if (m_Type == 13)
    rho = BrdqH(nDim, Xarg);
  else if (m_Type == 14)
    rho = IbaaH(nDim, Xarg);
  else if (m_Type == 15)
    rho = IbaqH(nDim, Xarg);
  else if (m_Type == 16)
    rho = Ibs1H(nDim, Xarg);
  else if (m_Type == 17)
    rho = Ibs2H(nDim, Xarg);
  else {
    cout << " TFDISTR::Density: !!!!!! WRONG m_Type= " << m_Type << endl;
  }
  return rho;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::BornH(int nDim, double *Xarg) {
  double BornXS(0.), BornXSpp(0.), BornXSpm(0.), BornXSmp(0.), BornXSmm(0.), BornXSdel(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.), QQpp(0.), QQpm(0.), QQmp(0.), QQmm(0.), QQdel(0.);
  double BornQ(0.), BornQpp(0.), BornQpm(0.), BornQmp(0.), BornQmm(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesBORN(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimBORN(Z))
    return BornXS;
  proc.GetFourMomentaBORN(Z, P_lab);
  if (proc.Cuts(P_lab))
    return BornXS;
  if (info.Iapol == 0) {
    proc.PdfConvBORN(Z, &QQ);
    proc.Born(Z, &BornQ);
    BornXS = 10. * BornQ * QQ * Jac;
  } else {
    proc.Bornpol(Z, &BornQpp, &BornQpm, &BornQmp, &BornQmm);
    if (info.PDFlib == -1) {
      BornXSpp = BornQpp * Jac;
      BornXSpm = BornQpm * Jac;
      BornXSmp = BornQmp * Jac;
      BornXSmm = BornQmm * Jac;
    } else {
      proc.PdfpolConvBORN(Z, &QQpp, &QQpm, &QQmp, &QQmm, &QQdel);
      BornXSpp = 10. * (BornQpp * QQpp + BornQpm * QQpm + BornQmp * QQmp + BornQmm * QQmm) * Jac;
      BornXSpm = 10. * (BornQpp * QQpm + BornQpm * QQpp + BornQmp * QQmm + BornQmm * QQmp) * Jac;
      BornXSmp = 10. * (BornQpp * QQmp + BornQpm * QQmm + BornQmp * QQpp + BornQmm * QQpm) * Jac;
      BornXSmm = 10. * (BornQpp * QQmm + BornQpm * QQmp + BornQmp * QQpm + BornQmm * QQpp) * Jac;
      BornXSdel= 10. * (BornQpp+BornQmm-BornQpm-BornQmp) * QQdel * Jac;
    }
  }
  if (info.Iapol == 1) {
    //sigma_NN
    BornXS = 0.25*(BornXSpp+BornXSpm+BornXSmp+BornXSmm);
  } else if (info.Iapol == 2) {
    //sigma_LL
    BornXS = 0.25*(BornXSpp-BornXSpm-BornXSmp+BornXSmm);
  } else if (info.Iapol == 3) {
    //sigma_LN
    BornXS = 0.25*(BornXSpp+BornXSpm-BornXSmp-BornXSmm);
  } else if (info.Iapol == 4) {
    //sigma_++
    BornXS = BornXSpp;
  } else if (info.Iapol == 5) {
    //sigma_+-
    BornXS = BornXSpm;
  } else if (info.Iapol == 6) {
    //sigma_-+
    BornXS = BornXSmp;
  } else if (info.Iapol == 7) {
    //sigma_--
    BornXS = BornXSmm;
  } else if (info.Iapol == 8) {
    //sigma_LL
    BornXS = 0.25*BornXSdel;
  } else if (info.Iapol == 9) {
    //sigma_NL
    BornXS = 0.25*(BornXSpp-BornXSpm+BornXSmp-BornXSmm);
  } else if (info.Iapol == 10) {
    //sigma_NL
    BornXS = 0.5*(BornXSmm-BornXSpp);
  } else if (info.Iapol == 11) {
    //sigma_NL
    BornXS = 0.5*(BornXSpp+BornXSmm);
  }

  proc.AFB(P_lab, &BornXS);
  if (BornXS < 0)
    proc.event.weightProc = -1.;
  BornXS = abs(BornXS);
//   cout << "GF = " << consts_.gf << endl;
  //--------------------------------------------------------------------------------------------------
  if (BornXS != BornXS) {
    BornXS = 0.;
    cout << "Born NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return BornXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::VirtH(int nDim, double *Xarg) {
  double VirtXS(0.), VirtXSpp(0.), VirtXSpm(0.), VirtXSmp(0.), VirtXSmm(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.), QQpp(0.), QQpm(0.), QQmp(0.), QQmm(0.), QQdel(0.);
  double VirtQ(0.), VirtQpp(0.), VirtQpm(0.), VirtQmp(0.), VirtQmm(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesBORN(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimBORN(Z))
    return VirtXS;
  proc.GetFourMomentaBORN(Z, P_lab);
  if (proc.Cuts(P_lab))
    return VirtXS;
  if (info.Iapol == 0) {
    proc.PdfConvBORN(Z, &QQ);
    proc.Virt(Z, &VirtQ);
    VirtXS = 10. * VirtQ * QQ * Jac;
  } else {
    proc.PdfpolConvBORN(Z, &QQpp, &QQpm, &QQmp, &QQmm, &QQdel);
    proc.Virtpol(Z, &VirtQpp, &VirtQpm, &VirtQmp, &VirtQmm);
    VirtXSpp = 10. * (VirtQpp * QQpp + VirtQpm * QQpm + VirtQmp * QQmp + VirtQmm * QQmm) * Jac;
    VirtXSpm = 10. * (VirtQpp * QQpm + VirtQpm * QQpp + VirtQmp * QQmm + VirtQmm * QQmp) * Jac;
    VirtXSmp = 10. * (VirtQpp * QQmp + VirtQpm * QQmm + VirtQmp * QQpp + VirtQmm * QQpm) * Jac;
    VirtXSmm = 10. * (VirtQpp * QQmm + VirtQpm * QQmp + VirtQmp * QQpm + VirtQmm * QQpp) * Jac;
  }
  if (info.Iapol == 1) {
    //sigma_NN
    VirtXS = 0.25*(VirtXSpp+VirtXSpm+VirtXSmp+VirtXSmm);
  } else if (info.Iapol == 2) {
    //sigma_LL
    VirtXS = 0.25*(VirtXSpp-VirtXSpm-VirtXSmp+VirtXSmm);
  } else if (info.Iapol == 3) {
    //sigma_LN
    VirtXS = 0.25*(VirtXSpp+VirtXSpm-VirtXSmp-VirtXSmm);
  } else if (info.Iapol == 4) {
    //sigma_++
    VirtXS = VirtXSpp;
  } else if (info.Iapol == 5) {
    //sigma_+-
    VirtXS = VirtXSpm;
  } else if (info.Iapol == 6) {
    //sigma_-+
    VirtXS = VirtXSmp;
  } else if (info.Iapol == 7) {
    //sigma_--
    VirtXS = VirtXSmm;
  }
  proc.AFB(P_lab, &VirtXS);
  if (VirtXS < 0)
    proc.event.weightProc = -1.;
  VirtXS = abs(VirtXS);
  //--------------------------------------------------------------------------------------------------
  if (VirtXS != VirtXS) {
    VirtXS = 0.;
    cout << "Virt NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return VirtXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::HardH(int nDim, double *Xarg) {
  double HardXS(0.), HardXSpp(0.), HardXSpm(0.), HardXSmp(0.), HardXSmm(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.), QQpp(0.), QQpm(0.), QQmp(0.), QQmm(0.);
  double HardQ(0.), HardQpp(0.), HardQpm(0.), HardQmp(0.), HardQmm(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesHARD(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimHARD(Z))
    return HardXS;
  proc.GetFourMomentaHARD(Z, P_lab, P_com);
  if (proc.Cuts(P_lab))
    return HardXS;
  setMomenta(proc.event);
  if (info.Iapol == 0) {
    proc.PdfConvHARD(Z, &QQ);
    proc.Hard(Z, &HardQ);
    HardXS = 20. * HardQ * QQ * Jac;
  } else {
    proc.Hardpol(Z, &HardQpp, &HardQpm, &HardQmp, &HardQmm);
    if (info.PDFlib == -1) {
      HardXSpp = HardQpp * Jac;
      HardXSpm = HardQpm * Jac;
      HardXSmp = HardQmp * Jac;
      HardXSmm = HardQmm * Jac;
    } else {
      proc.PdfpolConvHARD(Z, &QQpp, &QQpm, &QQmp, &QQmm);
      HardXSpp = 20. * (HardQpp * QQpp + HardQpm * QQpm + HardQmp * QQmp + HardQmm * QQmm) * Jac;
      HardXSpm = 20. * (HardQpp * QQpm + HardQpm * QQpp + HardQmp * QQmm + HardQmm * QQmp) * Jac;
      HardXSmp = 20. * (HardQpp * QQmp + HardQpm * QQmm + HardQmp * QQpp + HardQmm * QQpm) * Jac;
      HardXSmm = 20. * (HardQpp * QQmm + HardQpm * QQmp + HardQmp * QQpm + HardQmm * QQpp) * Jac;
    }
  }
  if (info.Iapol == 1) {
    //sigma_NN
    HardXS = 0.25*(HardXSpp+HardXSpm+HardXSmp+HardXSmm);
  } else if (info.Iapol == 2) {
    //sigma_LL
    HardXS = 0.25*(HardXSpp-HardXSpm-HardXSmp+HardXSmm);
  } else if (info.Iapol == 3) {
    //sigma_LN
    HardXS = 0.25*(HardXSpp+HardXSpm-HardXSmp-HardXSmm);
  } else if (info.Iapol == 4) {
    //sigma_++
    HardXS = HardXSpp;
  } else if (info.Iapol == 5) {
    //sigma_+-
    HardXS = HardXSpm;
  } else if (info.Iapol == 6) {
    //sigma_-+
    HardXS = HardXSmp;
  } else if (info.Iapol == 7) {
    //sigma_--
    HardXS = HardXSmm;
  }
  proc.AFB(P_lab, &HardXS);
  if (HardXS < 0)
    proc.event.weightProc = -1.;
  HardXS = abs(HardXS);
  //--------------------------------------------------------------------------------------------------
  if (HardXS != HardXS) {
    HardXS = 0.;
    cout << "Hard NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return HardXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::BrdqH(int nDim, double *Xarg) {
  double BrdqXS(0.), BrdqXSpp(0.), BrdqXSpm(0.), BrdqXSmp(0.), BrdqXSmm(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.), QQpp(0.), QQpm(0.), QQmp(0.), QQmm(0.);
  double BrdqQ(0.), BrdqQpp(0.), BrdqQpm(0.), BrdqQmp(0.), BrdqQmm(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesBRDQ(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimBORN(Z))
    return BrdqXS;
  proc.GetFourMomentaBRDQ(Z, P_lab);
  if (proc.Cuts(P_lab))
    return BrdqXS;
  if (info.Iapol == 0) {
    proc.PdfConvBRDQ(Z, &QQ);
    proc.Brdq(Z, &BrdqQ);
    BrdqXS = 10. * BrdqQ * QQ * Jac;
  } else {
    proc.PdfpolConvBRDQ(Z, &QQpp, &QQpm, &QQmp, &QQmm);
    proc.Brdqpol(Z, &BrdqQpp, &BrdqQpm, &BrdqQmp, &BrdqQmm);
    BrdqXSpp = 10. * (BrdqQpp * QQpp + BrdqQpm * QQpm + BrdqQmp * QQmp + BrdqQmm * QQmm) * Jac;
    BrdqXSpm = 10. * (BrdqQpp * QQpm + BrdqQpm * QQpp + BrdqQmp * QQmm + BrdqQmm * QQmp) * Jac;
    BrdqXSmp = 10. * (BrdqQpp * QQmp + BrdqQpm * QQmm + BrdqQmp * QQpp + BrdqQmm * QQpm) * Jac;
    BrdqXSmm = 10. * (BrdqQpp * QQmm + BrdqQpm * QQmp + BrdqQmp * QQpm + BrdqQmm * QQpp) * Jac;
  }
  if (info.Iapol == 1) {
    //sigma_NN
    BrdqXS = 0.25*(BrdqXSpp+BrdqXSpm+BrdqXSmp+BrdqXSmm);
  } else if (info.Iapol == 2) {
    //sigma_LL
    BrdqXS = 0.25*(BrdqXSpp-BrdqXSpm-BrdqXSmp+BrdqXSmm);
  } else if (info.Iapol == 3) {
    //sigma_LN
    BrdqXS = 0.25*(BrdqXSpp+BrdqXSpm-BrdqXSmp-BrdqXSmm);
  } else if (info.Iapol == 4) {
    //sigma_++
    BrdqXS = BrdqXSpp;
  } else if (info.Iapol == 5) {
    //sigma_+-
    BrdqXS = BrdqXSpm;
  } else if (info.Iapol == 6) {
    //sigma_-+
    BrdqXS = BrdqXSmp;
  } else if (info.Iapol == 7) {
    //sigma_--
    BrdqXS = BrdqXSmm;
  }
  proc.AFB(P_lab, &BrdqXS);
  if (BrdqXS < 0)
    proc.event.weightProc = -1.;
  BrdqXS = abs(BrdqXS);
  //--------------------------------------------------------------------------------------------------
  if (BrdqXS != BrdqXS) {
    BrdqXS = 0.;
    cout << "Brdq NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return BrdqXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::IbaaH(int nDim, double *Xarg) {
  double IbaaXS(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.);
  double Ibaa(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesIBAA(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimIBAA(Z))
    return IbaaXS;
  proc.GetFourMomentaIBAA(Z, P_lab);
  if (proc.Cuts(P_lab))
    return IbaaXS;
  proc.PdfConvIBAA(Z, &QQ);
  proc.Ibaa(Z, &Ibaa);
  IbaaXS = Ibaa * QQ * Jac;
  proc.AFB(P_lab, &IbaaXS);
  if (IbaaXS < 0)
    proc.event.weightProc = -1.;
  IbaaXS = abs(IbaaXS);
  //--------------------------------------------------------------------------------------------------
  if (IbaaXS != IbaaXS) {
    IbaaXS = 0.;
    cout << "Ibaa NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return IbaaXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::IbaqH(int nDim, double *Xarg) {
  double IbaqXS(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.);
  double IbaqQ(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesIBAQ(nDim, Xarg, Z, &Jac);
  if (info.Iqcd == 1) {
    masses_.mdn = info.Mdn / 10.;
    masses_.mup = info.Mup / 10.;
    r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
    r2masses_.rupm2 = masses_.mup * masses_.mup;
  }
  if (proc.CutsPrelimIBAQ(Z)) {
    if (info.Iqcd == 1) {
      masses_.mdn = info.Mdn;
      masses_.mup = info.Mup;
      r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
      r2masses_.rupm2 = masses_.mup * masses_.mup;
    }
    return IbaqXS;
  }
  proc.GetFourMomentaIBAQ(Z, P_lab);
  if (proc.Cuts(P_lab)) {
    if (info.Iqcd == 1) {
      masses_.mdn = info.Mdn;
      masses_.mup = info.Mup;
      r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
      r2masses_.rupm2 = masses_.mup * masses_.mup;
    }
    return IbaqXS;
  }
  proc.PdfConvIBAQ(Z, &QQ);
  proc.Ibaq(Z, &IbaqQ);
  IbaqXS = 40. * IbaqQ * QQ * Jac;
  proc.AFB(P_lab, &IbaqXS);
  if (IbaqXS < 0)
    proc.event.weightProc = -1.;
  IbaqXS = abs(IbaqXS);
  //--------------------------------------------------------------------------------------------------
  if (IbaqXS != IbaqXS) {
    IbaqXS = 0.;
    cout << "Ibaq NAN!" << endl;
  }
  if (info.Iqcd == 1) {
    masses_.mdn = info.Mdn;
    masses_.mup = info.Mup;
    r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
    r2masses_.rupm2 = masses_.mup * masses_.mup;
  }
  //--------------------------------------------------------------------------------------------------
  return IbaqXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::Ibs1H(int nDim, double *Xarg) {
  double Ibs1XS(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.);
  double Ibs1Q(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesIBS1(nDim, Xarg, Z, &Jac);
  if (info.Iqcd == 1) {
    masses_.mdn = info.Mdn / 10.;
    masses_.mup = info.Mup / 10.;
    r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
    r2masses_.rupm2 = masses_.mup * masses_.mup;
  }
  if (proc.CutsPrelimIBS1(Z)) {
    if (info.Iqcd == 1) {
      masses_.mdn = info.Mdn;
      masses_.mup = info.Mup;
      r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
      r2masses_.rupm2 = masses_.mup * masses_.mup;
    }
    return Ibs1XS;
  }
  proc.GetFourMomentaIBS1(Z, P_lab);
  if (proc.Cuts(P_lab)) {
    if (info.Iqcd == 1) {
      masses_.mdn = info.Mdn;
      masses_.mup = info.Mup;
      r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
      r2masses_.rupm2 = masses_.mup * masses_.mup;
    }
    return Ibs1XS;
  }
  proc.PdfConvIBS1(Z, &QQ);
  proc.Ibs1(Z, &Ibs1Q);
  Ibs1XS = 40. * Ibs1Q * QQ * Jac;
  proc.AFB(P_lab, &Ibs1XS);
  if (Ibs1XS < 0)
    proc.event.weightProc = -1.;
  Ibs1XS = abs(Ibs1XS);
  //--------------------------------------------------------------------------------------------------
  if (Ibs1XS != Ibs1XS) {
    Ibs1XS = 0.;
    cout << "Ibs1 NAN!" << endl;
  }
  if (info.Iqcd == 1) {
    masses_.mdn = info.Mdn;
    masses_.mup = info.Mup;
    r2masses_.rdnm2 = masses_.mdn * masses_.mdn;
    r2masses_.rupm2 = masses_.mup * masses_.mup;
  }
  //--------------------------------------------------------------------------------------------------
  return Ibs1XS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::Ibs2H(int nDim, double *Xarg) {
  double Ibs2XS(0.);
  double Z[nDim];
  double Jac(1.);
  double QQ(0.);
  double Ibs2Q(0.);
  proc.event.weightProc = 1.;
  //--------------------------------------------------------------------------------------------------
  proc.VariablesIBS2(nDim, Xarg, Z, &Jac);
  if (proc.CutsPrelimIBS2(Z))
    return Ibs2XS;
  proc.GetFourMomentaIBS2(Z, P_lab);
  if (proc.Cuts(P_lab))
    return Ibs2XS;
  proc.PdfConvIBS2(Z, &QQ);
  proc.Ibs2(Z, &Ibs2Q);
  Ibs2XS = 20. * Ibs2Q * QQ * Jac;
  proc.AFB(P_lab, &Ibs2XS);
  if (Ibs2XS < 0)
    proc.event.weightProc = -1.;
  Ibs2XS = abs(Ibs2XS);
  //--------------------------------------------------------------------------------------------------
  if (Ibs2XS != Ibs2XS) {
    Ibs2XS = 0.;
    cout << "Ibs2 NAN!" << endl;
  }
  //--------------------------------------------------------------------------------------------------
  return Ibs2XS;
}
